==> zeroeval-0.1.9-py3-none-any.whl: zeroeval/core/dataset_class.py <==
from typing import TYPE_CHECKING, List, Dict, Any, Optional, Union
from .writer import DatasetBackendWriter
from .reader import DatasetBackendReader

from .init import _validate_init
import os
import base64
from pathlib import Path
import mimetypes
import uuid
if TYPE_CHECKING:
    from .writer import DatasetWriter

class Dataset:
    """
    A class to represent a named collection of dictionary records.
    
    Attributes:
        name (str): The name of the dataset
        data (list): A list of dictionaries containing the data
        description (str): A description of the dataset
        backend_id (str): The ID of the dataset in the backend (after pushing)
        version_id (str): The ID of the dataset version in the backend
        version_number (int): The version number in the backend
    """
    
    def __init__(self, name: str, data: List[Dict[str, Any]], description: Optional[str] = None):
        """
        Initialize a Dataset with a name and data.
        
        Args:
            name (str): The name of the dataset
            data (list): A list of dictionaries containing the data
            description (str): A description of the dataset
            
        Raises:
            TypeError: If name is not a string or data is not a list
            ValueError: If any item in data is not a dictionary
        """
        if not isinstance(name, str):
            raise TypeError("Dataset name must be a string")
        
        if not isinstance(data, list):
            raise TypeError("Dataset data must be a list")
            
        if not all(isinstance(item, dict) for item in data):
            raise ValueError("All items in data must be dictionaries")
            
        self._name = name
        self._description = description
        # Keep the full rows (possibly including row_id, data, etc.)
        self._data = data.copy()  # avoid external modifications
        
        self._writer = DatasetBackendWriter()
        self._backend_id = None
        self._version_id = None
        self._version_number = None

    def add_rows(self, new_rows: List[Dict[str, Any]]) -> None:
        """
        Add one or more rows to the dataset.
        
        Args:
            new_rows (List[Dict[str, Any]]): The list of data rows to add.
            
        Raises:
            TypeError: If new_rows isn't a list or if any row isn't a dictionary.
        """
        if not isinstance(new_rows, list):
            raise TypeError("new_rows must be a list of dictionaries.")
        if not all(isinstance(row, dict) for row in new_rows):
            raise TypeError("All items in new_rows must be dictionaries.")
        
        self._data.extend(new_rows)

    def delete_row(self, index: int) -> None:
        """
        Delete a row from the dataset by index.
        
        Args:
            index (int): The index of the row to delete.
            
        Raises:
            IndexError: If the index is out of range.
        """
        try:
            del self._data[index]
        except IndexError:
            raise IndexError(f"Cannot delete row {index}, index is out of range.")

    def update_row(self, index: int, new_data: Dict[str, Any]) -> None:
        """
        Update a single row in the dataset by index, replacing its entire contents.
        
        Args:
            index (int): The index of the row to update.
            new_data (Dict[str, Any]): The new dictionary data for this row.
        
        Raises:
            TypeError: If new_data is not a dictionary.
            IndexError: If the index is out of range.
        """
        self.__setitem__(index, new_data)

    def push(self, create_new_version: bool = False):
        """
        Push the dataset to a storage destination.
        
        If a dataset with the same name already exists, a new version will be automatically created.
        
        Args:
            create_new_version (bool): For backward compatibility. This parameter is no longer needed
                as new versions are automatically created when a dataset name already exists.
                
        Returns:
            self: Returns self for method chaining
        """
        _validate_init()
        self._writer.write(self, create_new_version=create_new_version)
        return self

    @classmethod
    def pull(
        cls,
        dataset_name: str,
        version_number: Optional[int] = None,
        workspace_name: Optional[str] = None
    ) -> "Dataset":
        """
        Pull a dataset by dataset name, optionally specifying version_number.
        
        This uses the DatasetBackendReader to:
          1) Resolve workspace ID from API key
          2) Fetch metadata and rows from the backend
        """
        _validate_init()

        reader = DatasetBackendReader()
        # The reader will handle workspace resolution from API key internally
        return reader.pull_by_name(None, dataset_name, version_number=version_number)
    
    @property
    def version_id(self):
        """Get the backend version ID (if pushed)."""
        return self._version_id
    
    @property
    def version_number(self):
        """Get the backend version number (if pushed)."""
        return self._version_number
    
    @property
    def name(self):
        """Get the dataset name."""
        return self._name
    
    @property
    def description(self):
        """Get the dataset description."""
        return self._description
        
    @staticmethod
    def _encode_file_to_base64(file_path: str) -> str:
        """Convert a file to base64 encoding.
        
        Args:
            file_path: Path to the file to encode
            
        Returns:
            Base64 encoded string with appropriate data URI prefix
        """
        mime_type, _ = mimetypes.guess_type(file_path)
        if not mime_type:
            # Default to octet-stream if can't determine type
            mime_type = 'application/octet-stream'
            
        with open(file_path, 'rb') as file:
            encoded = base64.b64encode(file.read()).decode('utf-8')
            return f'data:{mime_type};base64,{encoded}'
    
    def add_image(self, row_index: int, column_name: str, image_path: str) -> None:
        """Add an image to a specific cell in the dataset.
        
        Args:
            row_index: Index of the row to update
            column_name: Name of the column to add the image to
            image_path: Path to the image file
            
        Raises:
            IndexError: If row_index is out of range
            FileNotFoundError: If image file does not exist
        """
        if not os.path.exists(image_path):
            raise FileNotFoundError(f"Image file not found: {image_path}")
            
        # Check if image path has valid extension
        valid_extensions = ['.jpg', '.jpeg', '.png', '.gif', '.webp']
        ext = Path(image_path).suffix.lower()
        if ext not in valid_extensions:
            raise ValueError(f"Invalid image file extension: {ext}. Must be one of {valid_extensions}")
            
        # Encode image as base64
        encoded_image = self._encode_file_to_base64(image_path)
        
        # Update cell with the encoded image
        try:
            row = self._data[row_index]
            if "data" in row:
                row["data"][column_name] = encoded_image
            else:
                row[column_name] = encoded_image
        except IndexError:
            raise IndexError(f"Row index {row_index} is out of range")
    
    def add_audio(self, row_index: int, column_name: str, audio_path: str) -> None:
        """Add an audio file to a specific cell in the dataset.
        
        Args:
            row_index: Index of the row to update
            column_name: Name of the column to add the audio to
            audio_path: Path to the audio file
            
        Raises:
            IndexError: If row_index is out of range
            FileNotFoundError: If audio file does not exist
        """
        if not os.path.exists(audio_path):
            raise FileNotFoundError(f"Audio file not found: {audio_path}")
            
        # Check if audio path has valid extension
        valid_extensions = ['.mp3', '.wav', '.ogg', '.m4a']
        ext = Path(audio_path).suffix.lower()
        if ext not in valid_extensions:
            raise ValueError(f"Invalid audio file extension: {ext}. Must be one of {valid_extensions}")
            
        # Encode audio as base64
        encoded_audio = self._encode_file_to_base64(audio_path)
        
        # Update cell with the encoded audio
        try:
            row = self._data[row_index]
            if "data" in row:
                row["data"][column_name] = encoded_audio
            else:
                row[column_name] = encoded_audio
        except IndexError:
            raise IndexError(f"Row index {row_index} is out of range")
    
    def add_video(self, row_index: int, column_name: str, video_path: str) -> None:
        """Add a video file to a specific cell in the dataset.
        
        Args:
            row_index: Index of the row to update
            column_name: Name of the column to add the video to
            video_path: Path to the video file
            
        Raises:
            IndexError: If row_index is out of range
            FileNotFoundError: If video file does not exist
        """
        if not os.path.exists(video_path):
            raise FileNotFoundError(f"Video file not found: {video_path}")
            
        # Check if video path has valid extension
        valid_extensions = ['.mp4', '.webm', '.mov']
        ext = Path(video_path).suffix.lower()
        if ext not in valid_extensions:
            raise ValueError(f"Invalid video file extension: {ext}. Must be one of {valid_extensions}")
            
        # Encode video as base64
        encoded_video = self._encode_file_to_base64(video_path)
        
        # Update cell with the encoded video
        try:
            row = self._data[row_index]
            if "data" in row:
                row["data"][column_name] = encoded_video
            else:
                row[column_name] = encoded_video
        except IndexError:
            raise IndexError(f"Row index {row_index} is out of range")
            
    def add_media_url(self, row_index: int, column_name: str, media_url: str, media_type: str) -> None:
        """Add a media URL to a specific cell in the dataset.
        
        Args:
            row_index: Index of the row to update
            column_name: Name of the column to add the media URL to
            media_url: URL pointing to the media file
            media_type: Type of media ('image', 'audio', or 'video')
            
        Raises:
            IndexError: If row_index is out of range
            ValueError: If media_type is invalid
        """
        valid_types = ['image', 'audio', 'video']
        if media_type not in valid_types:
            raise ValueError(f"Invalid media type: {media_type}. Must be one of {valid_types}")
            
        # Update cell with the media URL
        try:
            row = self._data[row_index]
            if "data" in row:
                row["data"][column_name] = media_url
            else:
                row[column_name] = media_url
        except IndexError:
            raise IndexError(f"Row index {row_index} is out of range")

    def __getitem__(self, idx: int) -> Dict[str, Any]:
        """
        By design, return *only* the 'data' part to the user,
        hiding row_id from direct indexing.
        """
        row = self._data[idx]
        # Return the data portion if present, else the row as-is
        return row["data"] if "data" in row else row
    
    def __setitem__(self, idx: int, value: Dict[str, Any]):
        """
        Updating a row at the given index. If the existing row has a row_id, preserve it.
        For direct user usage, the caller is presumably passing the 'data' portion.
        """
        if not isinstance(value, dict):
            raise TypeError("Each row must be a dictionary.")
        try:
            existing_row = self._data[idx]
            # If the existing row has a row_id, keep it
            if "row_id" in existing_row:
                # Wrap the new user data in {"data": value} if needed
                if "data" in value:
                    # They might have used the same structure. We'll trust them.
                    self._data[idx] = {"row_id": existing_row["row_id"], **value}
                else:
                    # They passed only data, so wrap it
                    self._data[idx] = {"row_id": existing_row["row_id"], "data": value}
            else:
                # If there is no row_id, just replace the entire row
                self._data[idx] = value
        except IndexError:
            raise IndexError(f"Cannot set row {idx}, index is out of range.")

    def __iter__(self):
        """
        By design, yield only the 'data' portion for iteration,
        so normal iteration does not expose row_id.
        """
        for row in self._data:
            yield row["data"] if "data" in row else row
            
    def __len__(self):
        """
        Returns the number of rows in the dataset.
        """
        return len(self._data)

    @property
    def data(self) -> List[Dict[str, Any]]:
        """
        Returns a list of just the data portion for each row.
        """
        return [row["data"] if "data" in row else row for row in self._data]
        
    @property
    def columns(self) -> List[str]:
        """
        Returns a list of column names in the dataset.
        """
        # Collect all unique keys from all rows
        columns = set()
        for row in self._data:
            if "data" in row and isinstance(row["data"], dict):
                columns.update(row["data"].keys())
            else:
                columns.update(row.keys())
                
        # Remove internal keys like 'row_id' if present
        if "row_id" in columns:
            columns.remove("row_id")
        
        return sorted(list(columns))



    # -------------------
    # Internal API: methods that return full rows.
    # Not intended for typical user usage, but for internal calls.
    # -------------------
    def _get_full_row(self, idx: int) -> Dict[str, Any]:
        """Return the entire row dictionary, including row_id, data, etc."""
        return self._data[idx]

    def _get_all_full_rows(self) -> List[Dict[str, Any]]:
        """Return all row dictionaries in their entirety."""
        return self._data

    def __delitem__(self, idx: int):
        """
        Allow deleting a row at the given index with del dataset[idx].
        """
        self.delete_row(idx)

    def __str__(self):
        """String representation of the dataset."""
        return f"Dataset('{self._name}', {len(self._data)} records)"
    
    def __repr__(self):
        """Detailed representation of the dataset."""
        return f"Dataset(name='{self._name}', size={len(self._data)}, columns={self.columns})"

==> zeroeval-0.2.11-py3-none-any.whl: zeroeval/core/dataset_class.py <==
import base64
import mimetypes
import os
from pathlib import Path
from typing import TYPE_CHECKING, Any, Optional

from .init import _validate_init
from .reader import DatasetBackendReader
from .writer import DatasetBackendWriter

if TYPE_CHECKING:
    pass

class Dataset:
    """
    A class to represent a named collection of dictionary records.
    
    Attributes:
        name (str): The name of the dataset
        data (list): A list of dictionaries containing the data
        description (str): A description of the dataset
        backend_id (str): The ID of the dataset in the backend (after pushing)
        version_id (str): The ID of the dataset version in the backend
        version_number (int): The version number in the backend
    """
    
    def __init__(self, name: str, data: list[dict[str, Any]], description: Optional[str] = None):
        """
        Initialize a Dataset with a name and data.
        
        Args:
            name (str): The name of the dataset
            data (list): A list of dictionaries containing the data
            description (str): A description of the dataset
            
        Raises:
            TypeError: If name is not a string or data is not a list
            ValueError: If any item in data is not a dictionary
        """
        if not isinstance(name, str):
            raise TypeError("Dataset name must be a string")
        
        if not isinstance(data, list):
            raise TypeError("Dataset data must be a list")
            
        if not all(isinstance(item, dict) for item in data):
            raise ValueError("All items in data must be dictionaries")
            
        self._name = name
        self._description = description
        # Keep the full rows (possibly including row_id, data, etc.)
        self._data = data.copy()  # avoid external modifications
        
        self._writer = DatasetBackendWriter()
        self._backend_id = None
        self._version_id = None
        self._version_number = None

    def add_rows(self, new_rows: list[dict[str, Any]]) -> None:
        """
        Add one or more rows to the dataset.
        
        Args:
            new_rows (List[Dict[str, Any]]): The list of data rows to add.
            
        Raises:
            TypeError: If new_rows isn't a list or if any row isn't a dictionary.
        """
        if not isinstance(new_rows, list):
            raise TypeError("new_rows must be a list of dictionaries.")
        if not all(isinstance(row, dict) for row in new_rows):
            raise TypeError("All items in new_rows must be dictionaries.")
        
        self._data.extend(new_rows)

    def delete_row(self, index: int) -> None:
        """
        Delete a row from the dataset by index.
        
        Args:
            index (int): The index of the row to delete.
            
        Raises:
            IndexError: If the index is out of range.
        """
        try:
            del self._data[index]
        except IndexError:
            raise IndexError(f"Cannot delete row {index}, index is out of range.")

    def update_row(self, index: int, new_data: dict[str, Any]) -> None:
        """
        Update a single row in the dataset by index, replacing its entire contents.
        
        Args:
            index (int): The index of the row to update.
            new_data (Dict[str, Any]): The new dictionary data for this row.
        
        Raises:
            TypeError: If new_data is not a dictionary.
            IndexError: If the index is out of range.
        """
        self.__setitem__(index, new_data)

    def push(self, create_new_version: bool = False):
        """
        Push the dataset to a storage destination.
        
        If a dataset with the same name already exists, a new version will be automatically created.
        
        Args:
            create_new_version (bool): For backward compatibility. This parameter is no longer needed
                as new versions are automatically created when a dataset name already exists.
                
        Returns:
            self: Returns self for method chaining
        """
        if not _validate_init():
            return self
        self._writer.write(self, create_new_version=create_new_version)
        return self

    @classmethod
    def pull(
        cls,
        dataset_name: str,
        version_number: Optional[int] = None,
        workspace_name: Optional[str] = None
    ) -> "Dataset":
        """
        Pull a dataset by dataset name, optionally specifying version_number.
        
        This uses the DatasetBackendReader to:
          1) Resolve workspace ID from API key
          2) Fetch metadata and rows from the backend
        """
        if not _validate_init():
            return None

        reader = DatasetBackendReader()
        # The reader will handle workspace resolution from API key internally
        return reader.pull_by_name(None, dataset_name, version_number=version_number)
    
    @property
    def version_id(self):
        """Get the backend version ID (if pushed)."""
        return self._version_id
    
    @property
    def version_number(self):
        """Get the backend version number (if pushed)."""
        return self._version_number
    
    @property
    def name(self):
        """Get the dataset name."""
        return self._name
    
    @property
    def description(self):
        """Get the dataset description."""
        return self._description
        
    @staticmethod
    def _encode_file_to_base64(file_path: str) -> str:
        """Convert a file to base64 encoding.
        
        Args:
            file_path: Path to the file to encode
            
        Returns:
            Base64 encoded string with appropriate data URI prefix
        """
        mime_type, _ = mimetypes.guess_type(file_path)
        if not mime_type:
            # Default to octet-stream if can't determine type
            mime_type = 'application/octet-stream'
            
        with open(file_path, 'rb') as file:
            encoded = base64.b64encode(file.read()).decode('utf-8')
            return f'data:{mime_type};base64,{encoded}'
    
    def add_image(self, row_index: int, column_name: str, image_path: str) -> None:
        """Add an image to a specific cell in the dataset.
        
        Args:
            row_index: Index of the row to update
            column_name: Name of the column to add the image to
            image_path: Path to the image file
            
        Raises:
            IndexError: If row_index is out of range
            FileNotFoundError: If image file does not exist
        """
        if not os.path.exists(image_path):
            raise FileNotFoundError(f"Image file not found: {image_path}")
            
        # Check if image path has valid extension
        valid_extensions = ['.jpg', '.jpeg', '.png', '.gif', '.webp']
        ext = Path(image_path).suffix.lower()
        if ext not in valid_extensions:
            raise ValueError(f"Invalid image file extension: {ext}. Must be one of {valid_extensions}")
            
        # Encode image as base64
        encoded_image = self._encode_file_to_base64(image_path)
        
        # Update cell with the encoded image
        try:
            row = self._data[row_index]
            if "data" in row:
                row["data"][column_name] = encoded_image
            else:
                row[column_name] = encoded_image
        except IndexError:
            raise IndexError(f"Row index {row_index} is out of range")
    
    def add_audio(self, row_index: int, column_name: str, audio_path: str) -> None:
        """Add an audio file to a specific cell in the dataset.
        
        Args:
            row_index: Index of the row to update
            column_name: Name of the column to add the audio to
            audio_path: Path to the audio file
            
        Raises:
            IndexError: If row_index is out of range
            FileNotFoundError: If audio file does not exist
        """
        if not os.path.exists(audio_path):
            raise FileNotFoundError(f"Audio file not found: {audio_path}")
            
        # Check if audio path has valid extension
        valid_extensions = ['.mp3', '.wav', '.ogg', '.m4a']
        ext = Path(audio_path).suffix.lower()
        if ext not in valid_extensions:
            raise ValueError(f"Invalid audio file extension: {ext}. Must be one of {valid_extensions}")
            
        # Encode audio as base64
        encoded_audio = self._encode_file_to_base64(audio_path)
        
        # Update cell with the encoded audio
        try:
            row = self._data[row_index]
            if "data" in row:
                row["data"][column_name] = encoded_audio
            else:
                row[column_name] = encoded_audio
        except IndexError:
            raise IndexError(f"Row index {row_index} is out of range")
    
    def add_video(self, row_index: int, column_name: str, video_path: str) -> None:
        """Add a video file to a specific cell in the dataset.
        
        Args:
            row_index: Index of the row to update
            column_name: Name of the column to add the video to
            video_path: Path to the video file
            
        Raises:
            IndexError: If row_index is out of range
            FileNotFoundError: If video file does not exist
        """
        if not os.path.exists(video_path):
            raise FileNotFoundError(f"Video file not found: {video_path}")
            
        # Check if video path has valid extension
        valid_extensions = ['.mp4', '.webm', '.mov']
        ext = Path(video_path).suffix.lower()
        if ext not in valid_extensions:
            raise ValueError(f"Invalid video file extension: {ext}. Must be one of {valid_extensions}")
            
        # Encode video as base64
        encoded_video = self._encode_file_to_base64(video_path)
        
        # Update cell with the encoded video
        try:
            row = self._data[row_index]
            if "data" in row:
                row["data"][column_name] = encoded_video
            else:
                row[column_name] = encoded_video
        except IndexError:
            raise IndexError(f"Row index {row_index} is out of range")
            
    def add_media_url(self, row_index: int, column_name: str, media_url: str, media_type: str) -> None:
        """Add a media URL to a specific cell in the dataset.
        
        Args:
            row_index: Index of the row to update
            column_name: Name of the column to add the media URL to
            media_url: URL pointing to the media file
            media_type: Type of media ('image', 'audio', or 'video')
            
        Raises:
            IndexError: If row_index is out of range
            ValueError: If media_type is invalid
        """
        valid_types = ['image', 'audio', 'video']
        if media_type not in valid_types:
            raise ValueError(f"Invalid media type: {media_type}. Must be one of {valid_types}")
            
        # Update cell with the media URL
        try:
            row = self._data[row_index]
            if "data" in row:
                row["data"][column_name] = media_url
            else:
                row[column_name] = media_url
        except IndexError:
            raise IndexError(f"Row index {row_index} is out of range")

    def __getitem__(self, idx: int) -> dict[str, Any]:
        """
        By design, return *only* the 'data' part to the user,
        hiding row_id from direct indexing.
        """
        row = self._data[idx]
        # Return the data portion if present, else the row as-is
        return row.get("data", row)
    
    def __setitem__(self, idx: int, value: dict[str, Any]):
        """
        Updating a row at the given index. If the existing row has a row_id, preserve it.
        For direct user usage, the caller is presumably passing the 'data' portion.
        """
        if not isinstance(value, dict):
            raise TypeError("Each row must be a dictionary.")
        try:
            existing_row = self._data[idx]
            # If the existing row has a row_id, keep it
            if "row_id" in existing_row:
                # Wrap the new user data in {"data": value} if needed
                if "data" in value:
                    # They might have used the same structure. We'll trust them.
                    self._data[idx] = {"row_id": existing_row["row_id"], **value}
                else:
                    # They passed only data, so wrap it
                    self._data[idx] = {"row_id": existing_row["row_id"], "data": value}
            else:
                # If there is no row_id, just replace the entire row
                self._data[idx] = value
        except IndexError:
            raise IndexError(f"Cannot set row {idx}, index is out of range.")

    def __iter__(self):
        """
        By design, yield only the 'data' portion for iteration,
        so normal iteration does not expose row_id.
        """
        for row in self._data:
            yield row.get("data", row)
            
    def __len__(self):
        """
        Returns the number of rows in the dataset.
        """
        return len(self._data)

    @property
    def data(self) -> list[dict[str, Any]]:
        """
        Returns a list of just the data portion for each row.
        """
        return [row.get("data", row) for row in self._data]
        
    @property
    def columns(self) -> list[str]:
        """
        Returns a list of column names in the dataset.
        """
        # Collect all unique keys from all rows
        columns = set()
        for row in self._data:
            if "data" in row and isinstance(row["data"], dict):
                columns.update(row["data"].keys())
            else:
                columns.update(row.keys())
                
        # Remove internal keys like 'row_id' if present
        if "row_id" in columns:
            columns.remove("row_id")
        
        return sorted(list(columns))



    # -------------------
    # Internal API: methods that return full rows.
    # Not intended for typical user usage, but for internal calls.
    # -------------------
    def _get_full_row(self, idx: int) -> dict[str, Any]:
        """Return the entire row dictionary, including row_id, data, etc."""
        return self._data[idx]

    def _get_all_full_rows(self) -> list[dict[str, Any]]:
        """Return all row dictionaries in their entirety."""
        return self._data

    def __delitem__(self, idx: int):
        """
        Allow deleting a row at the given index with del dataset[idx].
        """
        self.delete_row(idx)

    def __str__(self):
        """String representation of the dataset."""
        return f"Dataset('{self._name}', {len(self._data)} records)"
    
    def __repr__(self):
        """Detailed representation of the dataset."""
        return f"Dataset(name='{self._name}', size={len(self._data)}, columns={self.columns})"
